#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::folder;

use strict;
use warnings;

require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(newfolder renamefolder deletefolder movefolder);

our $VERSION = '0.01';

#######################################################################
our @ARGVBAACKUP = @ARGV;
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => [ ], 'cmdname'  => "");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
				 'user'	=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;
our $FolderName = undef;
our $NewFolderName = undef;
our $Force = 0;



#########################################################################

#########handle getopt warnings begin########################

sub InitParser_newfolder
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "new_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub InitParser_renamefolder
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "rename_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub InitParser_movefolder
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "move_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub InitParser_deletefolder
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "delete_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub CleanUpParser_deletefolder()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "delete_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;

}

sub CleanUpParser_newfolder()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "new_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;

}

sub CleanUpParser_renamefolder()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "rename_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;

}

sub CleanUpParser_movefolder()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "move_folder");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);

	$FolderName = undef;
	$NewFolderName = undef;
	$Force = 0;
	
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;

}

#########handle getopt warnings end#####################################

sub handle_help_usage
{
	my $opt = $_[0];
	
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}


sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_force_option
{
	$Force = 1;
}

sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions_newfolder()
{
	my $ret = undef;
	my $CoreOptRules = {
			'replace'   =>	\&handle_force_option 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions_deletefolder()
{
	my $ret = undef;
	my $CoreOptRules = {
			'replace'   =>	\&handle_force_option 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions_movefolder()
{
	my $ret = undef;
	my $CoreOptRules = {
			'replace'   =>	\&handle_force_option 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForCoreOptions_renamefolder()
{
	my $ret = undef;
	my $CoreOptRules = {
			'replace'   =>	\&handle_force_option 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|login|u|l:s'					=>	\&handle_login_data			
,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			
,
		'type:s'							=>	\&handle_recordtype_list	
		,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}


sub parseForUnknownOptions_newfolder
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;

	if($lmt == -1) 
	{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<folder name>",$CmdInfo{'cmdname'});
	}
	
	if($lmt > 0)
	{
		ThrowSyntaxErrorException("CQCLI_Repeat_FolderName","",$CmdInfo{'cmdname'});
	}
	
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if( $tmpARGV[$cnt] =~ /^(-)(with)/i)
		{
			print "Not supported";
		}
		else
		{
			$FolderName = $tmpARGV[$cnt];
		}
		
		$cnt = $cnt+1;
	}

	return 1;
}

sub parseForUnknownOptions_deletefolder
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if( $tmpARGV[$cnt] =~ /^(-)(with)/i)
		{
			print "Not supported";
		}
		else
		{
			$FolderName = $tmpARGV[$cnt];
		}
		
		$cnt = $cnt+1;
	}

	return 1;
}


sub parseForUnknownOptions_renamefolder
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;

	if($lmt == -1) 
	{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<source folder name> and <new folder name>",$CmdInfo{'cmdname'});
	}

	if($lmt == 0) 
	{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<new folder name>",$CmdInfo{'cmdname'});
	}
	
	if($lmt > 1)
	{
		ThrowSyntaxErrorException("CQCLI_Repeat_FolderName","",$CmdInfo{'cmdname'});
	}
	
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if( $tmpARGV[$cnt] =~ /^(-)(with)/i)
		{
			print "Not supported";
		}
		else
		{
			$FolderName = $tmpARGV[$cnt];
			$NewFolderName = $tmpARGV[$cnt+1];	
		}
		
		$cnt = $cnt+1;
	}

	return 1;
}


sub parseForUnknownOptions_movefolder
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;

	if($lmt == -1) 
	{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<source folder name> and <new folder name>",$CmdInfo{'cmdname'});
	}

	if($lmt == 0) 
	{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","<destination folder name>",$CmdInfo{'cmdname'});
	}
	
	if($lmt > 1)
	{
		ThrowSyntaxErrorException("CQCLI_Repeat_FolderName","",$CmdInfo{'cmdname'});
	}
	
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		if( $tmpARGV[$cnt] =~ /^(-)(with)/i)
		{
			print "Not supported";
		}
		else
		{
			$FolderName = $tmpARGV[$cnt];
			$NewFolderName = $tmpARGV[$cnt+1];
		}
		
		$cnt = $cnt+1;
	}

	return 1;
}

sub Parse_newfolder()
{
	try {

		parseForHelpUsage();
		parseForLoginCredentials();
		parseForCoreOptions_newfolder();
		parseForUnknownOptions_newfolder();
		
		$JavaReqObj->put("folder",$FolderName);
		
		if($Force) {
			$JavaReqObj->put("replace","true");
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Parse_deletefolder()
{
	try {

		parseForHelpUsage();
		parseForLoginCredentials();
		parseForCoreOptions_deletefolder();
		parseForUnknownOptions_deletefolder();
		
		$JavaReqObj->put("folder",$FolderName);
		
		if($Force) {
			$JavaReqObj->put("replace","true");
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Parse_renamefolder()
{
	try {

		parseForHelpUsage();
		parseForLoginCredentials();
		parseForCoreOptions_renamefolder();
		parseForUnknownOptions_renamefolder();
		
		$JavaReqObj->put("name",$FolderName);
		$JavaReqObj->put("newname",$NewFolderName);
		
		if($Force) {
			$JavaReqObj->put("replace","true");
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Parse_movefolder()
{
	try {

		parseForHelpUsage();
		parseForLoginCredentials();
		parseForCoreOptions_movefolder();
		parseForUnknownOptions_movefolder();
		
		$JavaReqObj->put("name",$FolderName);
		$JavaReqObj->put("newname",$NewFolderName);
		
		if($Force) {
			$JavaReqObj->put("replace","true");
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		return;
	}elsif($CliSession){
		return;
	}else{
		ValidateLoginCredentials();
		return;
	}

}

sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}

sub Validate()
{
	try {
		ValidateSessionInfo();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub newfolder
{
	my @tmpCmd = @_;
	my $req = undef;

	try {
		InitParser_newfolder(@tmpCmd);
		Parse_newfolder();
		Validate();
		$req =  GetJavaPayload("new_folder",$JavaReqObj);
		SetRequestObject($req);
		return 1;

	}finally {
		CleanUpParser_newfolder();
	};
}

sub deletefolder
{
	my @tmpCmd = @_;
	my $req = undef;
	
	try {
		InitParser_deletefolder(@tmpCmd);
		Parse_deletefolder();
		Validate();
		$req =  GetJavaPayload("delete_folder",$JavaReqObj);
		SetRequestObject($req);
		return 1;

	}finally {
		CleanUpParser_deletefolder();
	};
}

sub renamefolder
{
	my @tmpCmd = @_;
	my $req = undef;
	
	try {
		InitParser_renamefolder(@tmpCmd);
		Parse_renamefolder();
		Validate();
		$req =  GetJavaPayload("rename_folder",$JavaReqObj);
		SetRequestObject($req);
		return 1;

	}finally {
		CleanUpParser_renamefolder();
	};
}

sub movefolder
{
	my @tmpCmd = @_;
	my $req = undef;
	
	try {
		InitParser_movefolder(@tmpCmd);
		Parse_movefolder();
		Validate();
		$req =  GetJavaPayload("move_folder",$JavaReqObj);
		SetRequestObject($req);
		return 1;

	}finally {
		CleanUpParser_movefolder();
	};
}

1;

